from ccapis.apis.REST.bitfinex import BitfinexREST
from ccapis.apis.REST.bittrex import BittrexREST
from ccapis.apis.REST.poloniex import PoloniexREST
from ccapis.apis.REST.bithumb import BithumbREST
from ccapis.apis.REST.coinone import CoinoneREST
from ccapis.apis.REST.korbit import KorbitREST
# from ccapis.apis.REST.coinbase import CoinbaseREST
